@@ -15,6 +15,7 @@ from time import sleep
 import ffmpeg
 import humanize
 import pandas as pd
+import psutil
 import sqlite_utils
 from catt.api import CattDevice
 from rich.prompt import Confirm
@@ -23,6 +24,7 @@ from tabulate import tabulate
 from xklb.db import sqlite_con
 from xklb.utils import (
     CAST_NOW_PLAYING,
+    DEFAULT_MPV_SOCKET,
     FAKE_SUBTITLE,
     Pclose,
     Subcommand,
@@ -31,19 +33,26 @@ from xklb.utils import (
     cmd,
     cmd_interactive,
     filter_None,
+    flatten,
     get_ip_of_chromecast,
     get_ordinal_media,
     human_time,
     log,
+    mark_media_watched,
     mv_to_keep_folder,
+    os_bg_kwargs,
     remove_media,
     resize_col,
     tube_exclude_string,
     tube_include_string,
 )
 
+idle_mpv = lambda args: ["mpv", "--idle", f"--input-ipc-server={args.mpv_socket}"]
+
 DEFAULT_PLAY_QUEUE = 120
 
+pd.set_option("display.float_format", lambda x: "%.5f" % x)
+
 
 def override_sort(string):
     YEAR_MONTH = lambda var: f"cast(strftime('%Y%m',datetime({var} / 1000000000, 'unixepoch')) as int)"
@@ -137,6 +146,14 @@ Double spaces means one space
         action="store_true",
         help="lb tw -s 'something to delete' -rm",
     )
+    parser.add_argument(
+        "--mark-watched",
+        "--watched",
+        "--mark_played",
+        "--played",
+        action="store_true",
+        help="lb wt -s 'something to mark seen' --mark-watched",
+    )
 
     parser.add_argument(
         "--chromecast-device",
@@ -189,7 +206,7 @@ Double spaces means one space
         "--end", "-ve", help='Set the time to skip from the end of the media or use the magic word "dawsworth"'
     )
     parser.add_argument("--player", "-player", help='Override the default player; wt --player "vlc --vlc-opts"')
-    parser.add_argument("--mpv-socket", default="/tmp/mpv_socket")
+    parser.add_argument("--mpv-socket", default=DEFAULT_MPV_SOCKET)
 
     parser.add_argument(
         "--player-args-when-sub",
@@ -226,6 +243,9 @@ Double spaces means one space
     if args.sort:
         args.sort = " ".join(args.sort)
 
+    if args.print_column:
+        args.print_column = list(flatten([s.split(",") for s in args.print_column]))
+
     if args.duration:
         SEC_TO_M = 60
         duration_m = 0
@@ -298,21 +318,20 @@ def transcode(next_video):
     return next_video
 
 
-def delete_media(args, media_file):
+def delete_media(args, media_file: str):
     if len(args.prefix) > 0:
-        media_file.unlink()
-    elif which('trash-put') is not None:
+        Path(media_file).unlink()
+    elif which("trash-put") is not None:
         cmd("trash-put", media_file, strict=False)
     else:
-        media_file.unlink()
+        Path(media_file).unlink()
 
-    remove_media(args, str(media_file), quiet=True)
+    remove_media(args, media_file, quiet=True)
 
 
-def post_act(args, media_file):
+def post_act(args, media_file: str):
     if args.action in [Subcommand.listen, Subcommand.watch, Subcommand.tubelisten, Subcommand.tubewatch]:
-        args.con.execute("update media set play_count = play_count +1 where path = ?", (media_file,))
-        args.con.commit()
+        mark_media_watched(args, media_file)
 
     if args.post_action == "keep":
         pass
@@ -398,7 +417,7 @@ def listen_chromecast(args, m):
             catt_log = args.cc.play_url(m["path"], resolve=True, block=True)
     else:
         cast_process = subprocess.Popen(
-            ["catt", "-d", args.chromecast_device, "cast", "-s", FAKE_SUBTITLE, m["path"]], preexec_fn=os.setpgrp
+            ["catt", "-d", args.chromecast_device, "cast", "-s", FAKE_SUBTITLE, m["path"]], **os_bg_kwargs()
         )
         sleep(0.974)  # imperfect lazy sync; I use keyboard shortcuts to send `set speed` commands to mpv for resync
         # if pyChromecast provides a way to sync accurately that would be very interesting to know; I have not researched it
@@ -600,6 +619,7 @@ def construct_query(args):
         {', duration' if args.action != Subcommand.filesystem else ''}
         {', subtitle_count' if args.action == Subcommand.watch else ''}
         , size
+        {', ' + ', '.join(args.print_column) if args.print_column else ''}
         {', sparseness' if args.action == Subcommand.filesystem else ''}
         {', is_dir' if args.action == Subcommand.filesystem else ''}
         {', title' if args.action in [Subcommand.tubelisten, Subcommand.tubewatch] else ''}
@@ -634,8 +654,7 @@ def printer(args, query, bindings):
         from ({query}) """
 
     if args.print and "q" in args.print:
-        print(query)
-        exit()
+        return print(query)
 
     db_resp = pd.DataFrame([dict(r) for r in args.con.execute(query, bindings).fetchall()])
     db_resp.dropna(axis="columns", how="all", inplace=True)
@@ -652,9 +671,12 @@ def printer(args, query, bindings):
         exit(2)
 
     if args.delete:
-        remove_media(args, db_resp[["path"]].values.tolist(), quiet=True)
-        print(f"Deleted {len(db_resp)} media records")
-        exit()
+        print(f"Deleting {len(db_resp)} metadata records")
+        return remove_media(args, db_resp[["path"]].values.tolist(), quiet=True)
+
+    if args.mark_watched:
+        print(f"{len(db_resp)} metadata records marked watched")
+        return mark_media_watched(args, db_resp[["path"]].values.tolist())
 
     if "f" in args.print:
         if args.limit == 1:
@@ -664,7 +686,10 @@ def printer(args, query, bindings):
                 return printer(args, query, bindings)
             print(f)
         else:
-            for line in db_resp[["path"]].to_string(index=False, header=False).splitlines():
+            if not args.print_column:
+                args.print_column = ["path"]
+
+            for line in db_resp[[*args.print_column]].to_string(index=False, header=False).splitlines():
                 print(line.strip())
     else:
         tbl = db_resp.copy()
@@ -694,7 +719,16 @@ def printer(args, query, bindings):
             duration = human_time(duration)
             print("Total duration:", duration)
 
-    exit()
+
+def pkill(*command, strict=True):
+    found = 0
+    for process in psutil.process_iter():
+        if process.cmdline() == command:
+            process.terminate()
+            found = 1
+            break
+    if strict and found == 0:
+        raise Exception("Process not found")
 
 
 def process_actions(args):
@@ -702,40 +736,39 @@ def process_actions(args):
     query, bindings = construct_query(args)
 
     if args.print:
-        printer(args, query, bindings)
+        return printer(args, query, bindings)
 
     media = pd.DataFrame([dict(r) for r in args.con.execute(query, bindings).fetchall()])
     if len(media) == 0:
         print("No media found")
         exit(2)
 
+    if args.interdimensional_cable:
+        subprocess.Popen(idle_mpv(args))
+
     try:
         play(args, media)
     finally:
-        cmd("pkill", "-f", f"mpv --idle --input-ipc-server={args.mpv_socket}", strict=False)
+        if args.interdimensional_cable:
+            pkill(idle_mpv(args), strict=False)
         Path(args.mpv_socket).unlink(missing_ok=True)
+        if args.chromecast:
+            Path(CAST_NOW_PLAYING).unlink(missing_ok=True)
 
 
 def watch():
     args = parse_args("video.db", default_chromecast="Living Room TV")
     args.action = Subcommand.watch
-
     process_actions(args)
 
 
 def listen():
     args = parse_args("audio.db", default_chromecast="Xylo and Orchestra")
     args.action = Subcommand.listen
-
-    try:
-        process_actions(args)
-    finally:
-        if args.chromecast:
-            Path(CAST_NOW_PLAYING).unlink(missing_ok=True)
+    process_actions(args)
 
 
 def filesystem():
     args = parse_args("fs.db")
     args.action = Subcommand.filesystem
-
     process_actions(args)
