@@ -1,9 +1,10 @@
 import argparse
 import math
 import os
-from shutil import which
 import sys
 from pathlib import Path
+from shutil import which
+from sqlite3 import OperationalError
 from typing import Dict
 
 import ffmpeg
@@ -28,6 +29,7 @@ from xklb.utils import (
     log,
     remove_media,
     safe_unpack,
+    single_column_tolist,
 )
 
 
@@ -90,26 +92,31 @@ def parse_tags(mutagen: Dict, tinytag: Dict):
     return tags
 
 
+def calculate_sparseness(stat):
+    if stat.st_size == 0:
+        sparseness = 0
+    else:
+        blocks_allocated = stat.st_blocks * 512
+        sparseness = blocks_allocated / stat.st_size
+    return sparseness
+
+
 def extract_metadata(args, f):
     try:
         stat = os.stat(f)
     except:
         return
 
-    if stat.st_size == 0:
-        sparseness = 0
-    else:
-        blocks_allocated = stat.st_blocks * 512
-        sparseness = blocks_allocated / stat.st_size
-
     media = dict(
         path=f,
         size=stat.st_size,
-        sparseness=sparseness,
         time_created=int(stat.st_ctime),
         time_modified=int(stat.st_mtime),
     )
 
+    if hasattr(stat, "st_blocks"):
+        media = {**media, "sparseness": calculate_sparseness(stat)}
+
     if args.db_type == "f":
         media = {**media, "is_dir": os.path.isdir(f)}
 
@@ -121,7 +128,7 @@ def extract_metadata(args, f):
         except:
             print(f"Failed reading {f}", file=sys.stderr)
             if args.delete_unplayable:
-                if which('trash-put') is not None:
+                if which("trash-put") is not None:
                     cmd("trash-put", f, strict=False)
                 else:
                     Path(f).unlink()
@@ -223,13 +230,21 @@ def extract_metadata(args, f):
     return media
 
 
+def get_columns(args):
+    try:
+        query = "SELECT name FROM PRAGMA_TABLE_INFO('media') where type in ('TEXT', 'INTEGER');"
+        cols = single_column_tolist(args.con.execute(query).fetchall(), "name")
+    except OperationalError:
+        cols = []
+    return cols
+
+
 def optimize_db(args):
     print("Optimizing database")
     if Path(args.database).exists():
         cmd("sqlite-utils", "optimize", args.database)
-        columns = cmd(
-            f"sqlite-utils tables {args.database} --columns | jq -r '.[0].columns[]'", shell=True
-        ).stdout.splitlines()
+        columns = get_columns(args)
+
         for column in columns:
             cmd("sqlite-utils", "create-index", "--if-not-exists", "--analyze", args.database, "media", column)
 
@@ -360,6 +375,7 @@ def main(args=None):
 
     if args.overwrite_db:
         Path(args.database).unlink(missing_ok=True)
+        Path(args.database).touch()
 
     extractor(args)
 
