@@ -33,6 +33,7 @@ else:
 SQLITE_PARAM_LIMIT = 32765
 FAKE_SUBTITLE = os.path.join(gettempdir(), "sub.srt")  # https://github.com/skorokithakis/catt/issues/393
 CAST_NOW_PLAYING = os.path.join(gettempdir(), "catt_playing")
+DEFAULT_MPV_SOCKET = os.path.join(gettempdir(), "mpv_socket")
 
 
 class Subcommand:
@@ -174,7 +175,7 @@ def chunks(lst, n):
         yield lst[i : i + n]
 
 
-def flatten(xs):
+def flatten(xs: Iterable):
     for x in xs:
         if isinstance(x, Iterable) and not isinstance(x, (str, bytes)):
             yield from flatten(x)
@@ -184,7 +185,7 @@ def flatten(xs):
             yield x
 
 
-def conform(list_):
+def conform(list_: Union[str, Iterable]):
     if not isinstance(list_, list):
         list_ = [list_]
     list_ = flatten(list_)
@@ -192,7 +193,7 @@ def conform(list_):
     return list_
 
 
-def remove_media(args, deleted_files, quiet=False):
+def remove_media(args, deleted_files: Union[str, list], quiet=False):
     deleted_files = conform(deleted_files)
     if len(deleted_files) > 0:
         if not quiet:
@@ -210,6 +211,18 @@ def remove_media(args, deleted_files, quiet=False):
             args.con.commit()
 
 
+def mark_media_watched(args, files):
+    files = conform(files)
+    if len(files) > 0:
+        df_chunked = chunks(files, SQLITE_PARAM_LIMIT)
+        for l in df_chunked:
+            args.con.execute(
+                "update media set play_count = play_count +1 where path in (" + ",".join(["?"] * len(l)) + ")",
+                (*l,),
+            )
+            args.con.commit()
+
+
 def get_media_files(path, audio=False):
     FFMPEG_DEMUXERS = (
         "str|aa|aax|acm|adf|adp|dtk|ads|ss2|adx|aea|afc|aix|al|apl"
@@ -243,6 +256,16 @@ def get_media_files(path, audio=False):
     return video_files
 
 
+def os_bg_kwargs():
+    if hasattr(os, "setpgrp"):
+        os_kwargs = dict(start_new_session=True)
+    else:
+        CREATE_NEW_PROCESS_GROUP = 0x00000200
+        DETACHED_PROCESS = 0x00000008
+        os_kwargs = dict(creationflags=DETACHED_PROCESS | CREATE_NEW_PROCESS_GROUP)
+    return os_kwargs
+
+
 def cmd(*command, strict=True, cwd=None, quiet=True, interactive=False, **kwargs):
     EXP_FILTER = re.compile(
         "|".join(
@@ -273,10 +296,9 @@ def cmd(*command, strict=True, cwd=None, quiet=True, interactive=False, **kwargs
     if len(command) == 1 and kwargs.get("shell") is True:
         command = command[0]
 
-    r = subprocess.run(
-        command, capture_output=True, text=True, cwd=cwd, preexec_fn=None if interactive else os.setpgrp, **kwargs
-    )
-    # TODO Windows support: creationflags=subprocess.DETACHED_PROCESS | subprocess.CREATE_NEW_PROCESS_GROUP
+    os_kwargs = {} if interactive else os_bg_kwargs()
+    r = subprocess.run(command, capture_output=True, text=True, cwd=cwd, **os_kwargs, **kwargs)
+
     log.debug(r.args)
     r.stdout = print_std(r.stdout)
     r.stderr = print_std(r.stderr)
@@ -429,11 +451,10 @@ class argparse_dict(argparse.Action):
         setattr(args, self.dest, d)
 
 
-TERMINAL_SIZE = SimpleNamespace(columns=80, lines=60)
 try:
-    os.get_terminal_size()
+    TERMINAL_SIZE = os.get_terminal_size()
 except:
-    pass
+    TERMINAL_SIZE = SimpleNamespace(columns=80, lines=60)
 
 
 def resize_col(tbl, col, size=10):
